using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Weerstation
{
    /// <summary>
    ///  The thermometre class uses the gauge as base class to derive its own functionalities. We only musty draw things here
    /// </summary>
    class VerticalGauge:Gauge
    {
        //---------------------------------------------------------------------------------------
        protected override void DrawGauge(System.Windows.Media.DrawingContext drawingContext, double min, double max, double currentvalue)
        {
            Brush drawingbrush = this.GaugeColor;            
            if (drawingbrush != null)
            {    
                // the scale starts at 1/3 of the lower graph, so start there for our drawing
                double yratio = (currentvalue-min) / (max - min) * (this.ActualHeight);
                if (yratio > this.ActualHeight) yratio = this.ActualHeight;
                if (yratio < 0) yratio = 0;
                drawingContext.DrawRectangle(drawingbrush, null, new Rect(new Point(0, this.ActualHeight), new Point(this.ActualWidth, this.ActualHeight - yratio)));                 
            }            
        }
        //-------------------------------------------------------------------------------------
        protected override void DrawGrid(System.Windows.Media.DrawingContext drawingContext, double min, double max)
        {
            int majorlines = this.MajorDivisions;
            int minorlines = this.SubDivisions;

            double yscale = this.ActualHeight/majorlines;
            double y_scale_minor = yscale / minorlines;

            for (int i = 0; i <= majorlines; i++) // draw the lajor lines
            {
                drawingContext.DrawLine(new Pen(this.GridColor, 2), new Point(0, yscale * i), new Point(this.ActualWidth,yscale*i));
                if (i < majorlines)
                {
                    for (int j = 0; j < minorlines; j++) // draw the minor lines
                    {
                        drawingContext.DrawLine(new Pen(this.GridColor, 1), new Point(0, y_scale_minor * j + i * yscale), new Point(this.ActualWidth, yscale * i + y_scale_minor * j));
                    }
                }
            }
        }
        //-------------------------------------------------------------------------------------
        protected override void DrawAxis(System.Windows.Media.DrawingContext drawingContext, double min, double max)
        {
            FormattedText ftext;
            int majorlines = this.MajorDivisions;
            double minval = this.GaugetMin;
            double maxval = this.GaugeMax;
            double stepsize = (maxval - minval) / (majorlines);
            double yscale = this.ActualHeight / majorlines;
            for (int i = 0; i <= majorlines; i++)
            {
                if (i == majorlines) // draw the lable on the axis
                {
                    ftext = new FormattedText(this.Lable, new System.Globalization.CultureInfo("en-US"), FlowDirection.LeftToRight, new Typeface("Arial"), this.Fontsize, this.Foreground);
                    drawingContext.DrawText(ftext, new Point(-ftext.Width - 1, this.ActualHeight - (yscale * i + ftext.Height / 2)));

                }
                else
                {
                    ftext = new FormattedText((stepsize * i+min).ToString("#;-#;0"), new System.Globalization.CultureInfo("en-US"), FlowDirection.LeftToRight, new Typeface("Arial"), this.Fontsize, this.Foreground);
                    drawingContext.DrawText(ftext, new Point(-ftext.Width - 1, this.ActualHeight - (yscale * i + ftext.Height / 2)));
                
                }
            }
        }
        //-------------------------------------------------------------------------------------
        protected override void DrawOverlay(System.Windows.Media.DrawingContext drawingContext)
        {
            // no overlay desired  
        }
        //-------------------------------------------------------------------------------------
    }
}
