using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

// gauge to indicate the current state of measurements


namespace Weerstation
{
    public abstract class Gauge : FrameworkElement
    {
        private VisualCollection _children;
        private DrawingVisual background_layer;
        private DrawingVisual grid_drawing_layer;
        private DrawingVisual Gauge_drawing_layer;
        private DrawingVisual axis_drawing_layer;
        private DrawingVisual overlay_layer;

        private Brush backgroundbrush_;             // brush to define the background
        private Brush gridbrush_;                   // the color of our grid
        private Brush forebrush_;                   // the color of our font (numbers and lables)
        private Brush Gaugebrush_;                  // the color of our graph

        private double min_value_;                  // keep the absolute minimum value of the past measurements
        private double max_value_;                  // keep the abolute maximum of the past measurements
        private double current_value_;              // keep the current value
        private double Gauge_min_;                  // the minimal value range of the Gauge
        private double Gauge_max_;                  // the maximal value range of the Gauge
        private string gauge_lable_;                // the horizontal unit to display on the graph       
        private int major_divisions_;               // the amount of divisions   
        private int sub_dibivions_;                 // the amount of divisions in each major division  
        private double fontsize_;                   // keep the size of our char font 
        //----------------------------------------------------------------------------------------
        static Gauge()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(Gauge), new FrameworkPropertyMetadata(typeof(Gauge)));
        }
        //----------------------------------------------------------------------------------------
        public Gauge()
        {
            _children = new VisualCollection(this);
            background_layer = new DrawingVisual();
            grid_drawing_layer = new DrawingVisual();
            Gauge_drawing_layer = new DrawingVisual();
            axis_drawing_layer = new DrawingVisual();
            overlay_layer = new DrawingVisual();
            //- add the layers to the visuals
            _children.Add(background_layer);
            _children.Add(Gauge_drawing_layer);
            _children.Add(grid_drawing_layer);
            _children.Add(overlay_layer);
            _children.Add(axis_drawing_layer);            
            // default settings
            min_value_ = 0;
            max_value_ = 0;
            Gauge_max_ = 100;
            Gauge_min_ = 0;
            current_value_ = 50;
            gauge_lable_ = "T (C)";
            major_divisions_ = 5;
            sub_dibivions_ = 5;
            fontsize_ = 14;
        }
        //----------------------------------------------------------------------------------------
        public void setNewValue(double value)
        {
            this.current_value_ = value;
            this.RedrawGauge();
        }
        //----------------------------------------------------------------------------------------
        protected abstract void DrawGauge(DrawingContext drawingContext,double min, double max,double currentvalue);
        protected abstract void DrawGrid(DrawingContext drawingContext, double min, double max);
        protected abstract void DrawAxis(DrawingContext drawingContext, double min, double max);
        protected abstract void DrawOverlay(DrawingContext drawingContext);
        //----------------------------------------------------------------------------------------
        private void DrawGauge()
        {
            DrawingContext drawingContext = Gauge_drawing_layer.RenderOpen();
            DrawGauge(drawingContext, Gauge_min_, Gauge_max_, current_value_);
            drawingContext.Close();
            Gauge_drawing_layer.Clip = new RectangleGeometry(new Rect(0, 0, this.ActualWidth, this.ActualHeight));            
        }
        //----------------------------------------------------------------------------------------
        private void DrawGrid() // draw our grid in the background
        {
            DrawingContext drawingContext = grid_drawing_layer.RenderOpen();
            DrawGrid(drawingContext, Gauge_min_, Gauge_max_);
            drawingContext.Close();
        }
        //----------------------------------------------------------------------------------------
        private void DrawAxis()
        {
            DrawingContext drawingContext = axis_drawing_layer.RenderOpen();
            DrawAxis(drawingContext, Gauge_min_, Gauge_max_);
            drawingContext.Close();
        }
        //----------------------------------------------------------------------------------------
        private void DrawOverlay()
        {
            DrawingContext drawingContext = overlay_layer.RenderOpen();
            DrawOverlay(drawingContext);
            drawingContext.Close();
        }
        //---------------------------------------------------------------------------------------
        private void DrawBackground()
        {
            if (this.backgroundbrush_ != null)
            {
                DrawingContext drawingContext = background_layer.RenderOpen();
                drawingContext.DrawRectangle(backgroundbrush_, null, new Rect(new Point(0, 0), new Point(this.ActualWidth, this.ActualHeight)));
                drawingContext.Close();
            }
        }
        //---------------------------------------------------------------------------------------
        protected void RedrawGauge()
        {
            this.DrawBackground();
            this.DrawGauge();
            this.DrawOverlay();
            this.DrawGrid();
            this.DrawAxis();
        }
        //----------------------------------------------------------------------------------------
        // Provide a required override for the VisualChildrenCount property.
        protected override int VisualChildrenCount
        {
            get { return _children.Count; }
        }
        //----------------------------------------------------------------------------------------
        // Provide a required override for the GetVisualChild method.
        protected override Visual GetVisualChild(int index)
        {
            if (index < 0 || index >= _children.Count)
            {
                throw new ArgumentOutOfRangeException();
            }

            return _children[index];
        }
        //----------------------------------------------------------------------------------------
        //--------------------- properties -------------------------------------------------------
        //--------------------------------------------------------------------------------------------
        public Brush Background
        {
            get { return (Brush)GetValue(BackgroundProperty); }
            set { SetValue(BackgroundProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public Brush Foreground
        {
            get { return (Brush)GetValue(ForegroundProperty); }
            set { SetValue(ForegroundProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public Brush GridColor
        {
            get { return (Brush)GetValue(GridColorProperty); }
            set { SetValue(GridColorProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public Brush GaugeColor
        {
            get { return (Brush)GetValue(GaugeColorProperty); }
            set { SetValue(GaugeColorProperty, value); }
        }       
        //--------------------------------------------------------------------------------------------
        public int MajorDivisions
        {
            get { return (int)GetValue(MajorDivisionsProperty); }
            set { SetValue(MajorDivisionsProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public int SubDivisions
        {
            get { return (int)GetValue(SubDivisionsProperty); }
            set { SetValue(SubDivisionsProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public int Fontsize
        {
            get { return (int)GetValue(FontsizeProperty); }
            set { SetValue(FontsizeProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public string Lable
        {
            get { return (string)GetValue(LableProperty); }
            set { SetValue(LableProperty, value); }
        }       
        //--------------------------------------------------------------------------------------------
        public double GaugetMin
        {
            get { return (double)GetValue(GaugeMinProperty); }
            set { SetValue(GaugeMinProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public double GaugeMax
        {
            get { return (double)GetValue(GaugeMaxProperty); }
            set { SetValue(GaugeMaxProperty, value); }
        }
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty BackgroundProperty =
        DependencyProperty.RegisterAttached("Background", typeof(Brush), typeof(Gauge), new FrameworkPropertyMetadata(null, OnBackgroundPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty ForegroundProperty =
        DependencyProperty.RegisterAttached("Foreground", typeof(Brush), typeof(Gauge), new FrameworkPropertyMetadata(null, OnForegroundPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty GridColorProperty =
        DependencyProperty.RegisterAttached("GridColor", typeof(Brush), typeof(Gauge), new FrameworkPropertyMetadata(null, OnGridColorPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty GaugeColorProperty =
        DependencyProperty.RegisterAttached("GaugeColor", typeof(Brush), typeof(Gauge), new FrameworkPropertyMetadata(null, OnGaugeColorPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty MajorDivisionsProperty =
        DependencyProperty.RegisterAttached("MajorDivisions", typeof(int), typeof(Gauge), new FrameworkPropertyMetadata(-1, OnMajorDivisionsPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty SubDivisionsProperty =
        DependencyProperty.RegisterAttached("SubDivisions", typeof(int), typeof(Gauge), new FrameworkPropertyMetadata(-1, OnSubDivisionsPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty FontsizeProperty =
        DependencyProperty.RegisterAttached("Fontsize", typeof(int), typeof(Gauge), new FrameworkPropertyMetadata(12, OnFontsizePropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty LableProperty =
        DependencyProperty.RegisterAttached("Lable", typeof(string), typeof(Gauge), new FrameworkPropertyMetadata("T (C)", OnLablePropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty GaugeMinProperty =
        DependencyProperty.RegisterAttached("GaugeMin", typeof(double), typeof(Gauge), new FrameworkPropertyMetadata(-100.0, OnGaugeMinPropertyChanged));
        //--------------------------------------------------------------------------------------------
        public static DependencyProperty GaugeMaxProperty =
        DependencyProperty.RegisterAttached("GaugeMax", typeof(double), typeof(Gauge), new FrameworkPropertyMetadata(100.0, OnGaugeMaxPropertyChanged));
        //--------------------------------------------------------------------------------------------        
        private static void OnBackgroundPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            Brush background = (Brush)e.NewValue;
            // Put some update logic here...
            control.backgroundbrush_ = background;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------        
        private static void OnForegroundPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            Brush foreground = (Brush)e.NewValue;
            // Put some update logic here...
            if (foreground != null) control.forebrush_ = foreground;
            else control.forebrush_ = Brushes.Black;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------        
        private static void OnGridColorPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            Brush gridcolor = (Brush)e.NewValue;
            // Put some update logic here...
            if (gridcolor != null) control.gridbrush_ = gridcolor;
            else control.gridbrush_ = Brushes.Black;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------        
        private static void OnGaugeColorPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            Brush Gaugecolor = (Brush)e.NewValue;
            // Put some update logic here...
            if (Gaugecolor != null) control.Gaugebrush_ = Gaugecolor;
            else control.Gaugebrush_ = Brushes.Black;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------        
        private static void OnMajorDivisionsPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            int divisions = (int)e.NewValue;
            // Put some update logic here...
            if (divisions < 1) divisions = 1;
            control.major_divisions_ = divisions;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------        
        private static void OnSubDivisionsPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            int divisions = (int)e.NewValue;
            // Put some update logic here...
            if (divisions < 1) divisions = 1;
            control.sub_dibivions_ = divisions;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------       
        private static void OnFontsizePropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            int fontsize = (int)e.NewValue;
            // Put some update logic here...
            if (fontsize < 1) fontsize = 1;
            control.fontsize_ = fontsize;
            control.RedrawGauge();
        }        
        //--------------------------------------------------------------------------------------------       
        private static void OnGaugeMinPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            control.Gauge_min_ = (double)e.NewValue;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------       
        private static void OnGaugeMaxPropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            control.Gauge_max_ = (double)e.NewValue;
            control.RedrawGauge();
        }
        //--------------------------------------------------------------------------------------------       
        private static void OnLablePropertyChanged(DependencyObject source, DependencyPropertyChangedEventArgs e)
        {
            Gauge control = source as Gauge;
            string lable = (string)e.NewValue;
            // Put some update logic here...
            if (lable != null)
            {
                control.gauge_lable_ = lable;
                control.RedrawGauge();
            }
            else
            {
                // throw exception
            }
        }        
    }
}
